// RunAsSystem


#ifndef __LOCAL_VALUE_H__
#define __LOCAL_VALUE_H__


class auto_handle {
private:
	HANDLE handle;
public:
	explicit auto_handle(HANDLE handle = NULL)
			: handle(handle) {
	}
	~auto_handle() {
		if ((handle != NULL) && (handle != INVALID_HANDLE_VALUE)) {
			::CloseHandle(handle);
		}
	}
	operator HANDLE() {
		return handle;
	}
	HANDLE* operator&() {
		return &handle;
	}
};


template<typename Type>
class auto_LocalHeap {
private:
	Type* ptr;
public:
	explicit auto_LocalHeap(Type* ptr = NULL)
			: ptr(ptr) {
	}
	~auto_LocalHeap() {
		if (ptr != NULL) {
			::LocalFree(ptr);
		}
	}
	operator Type*() {
		return ptr;
	}
	Type** operator&() {
		return &ptr;
	}
};


class auto_SID {
private:
	PSID sid;
public:
	explicit auto_SID(SID* sid = NULL)
			: sid(sid) {
	}
	~auto_SID() {
		if (sid != NULL) {
			::FreeSid(sid);
		}
	}
	operator PSID() {
		return sid;
	}
	PSID* operator&() {
		return &sid;
	}
};


template<typename Type>
class auto_buffer {
private:
	Type* ptr;
public:
	explicit auto_buffer(DWORD size)
			: ptr(NULL) {
		if (size > 0) {
			ptr = static_cast<Type*>(::VirtualAlloc(NULL, size, MEM_COMMIT, PAGE_READWRITE));
			if (ptr == NULL) {
				fatalError(L"::VirtualAlloc() failed", ::GetLastError());
			}
		}
	}
	~auto_buffer() {
		if (ptr != NULL) {
			::VirtualFree(ptr, 0, MEM_RELEASE);
		}
	}
	operator Type*() {
		return ptr;
	}
	Type* operator->() {
		return ptr;
	}
};


class EnvironmentBlock {
private:
	void* env;
public:
	EnvironmentBlock(HANDLE token)
			: env(NULL) {
		BOOL result = ::CreateEnvironmentBlock(&env, token, FALSE);
		if (!result) {
			DWORD lastError = ::GetLastError();
			if (lastError == ERROR_ENVVAR_NOT_FOUND) {
				// thru
				env = NULL;
			} else {
				fatalError(L"::CreateEnvironmentBlock() failed", lastError);
			}
		}
	}
	~EnvironmentBlock() {
		if (env != NULL) {
			::DestroyEnvironmentBlock(env);
		}
	}
	operator void*() {
		return env;
	}
};


#endif // __LOCAL_VALUE_H__
